@@ -19,13 +19,13 @@ from __future__ import annotations
 import asyncio
 import json
 import time
+import warnings
 from enum import Enum
 from functools import cached_property, wraps
 from inspect import signature
 from typing import TYPE_CHECKING, Any, Callable, Sequence, Set, TypeVar, cast
 
 import aiohttp
-from aiohttp import ClientResponseError
 from asgiref.sync import sync_to_async
 from requests.auth import AuthBase
 from requests.sessions import Session
@@ -39,6 +39,8 @@ if TYPE_CHECKING:
 
     from airflow.models import Connection
 
+DBT_CAUSE_MAX_LENGTH = 255
+
 
 def fallback_to_default_account(func: Callable) -> Callable:
     """
@@ -179,9 +181,12 @@ class DbtCloudHook(HttpHook):
     def get_ui_field_behaviour(cls) -> dict[str, Any]:
         """Build custom field behavior for the dbt Cloud connection form in the Airflow UI."""
         return {
-            "hidden_fields": ["schema", "port", "extra"],
+            "hidden_fields": ["schema", "port"],
             "relabeling": {"login": "Account ID", "password": "API Token", "host": "Tenant"},
-            "placeholders": {"host": "Defaults to 'cloud.getdbt.com'."},
+            "placeholders": {
+                "host": "Defaults to 'cloud.getdbt.com'.",
+                "extra": "Optional JSON-formatted extra.",
+            },
         }
 
     def __init__(self, dbt_cloud_conn_id: str = default_conn_name, *args, **kwargs) -> None:
@@ -192,6 +197,10 @@ class DbtCloudHook(HttpHook):
     def _get_tenant_domain(conn: Connection) -> str:
         return conn.host or "cloud.getdbt.com"
 
+    @staticmethod
+    def _get_proxies(conn: Connection) -> dict[str, str] | None:
+        return conn.extra_dejson.get("proxies", None)
+
     @staticmethod
     def get_request_url_params(
         tenant: str, endpoint: str, include_related: list[str] | None = None, *, api_version: str = "v2"
@@ -235,14 +244,18 @@ class DbtCloudHook(HttpHook):
         endpoint = f"{account_id}/runs/{run_id}/"
         headers, tenant = await self.get_headers_tenants_from_connection()
         url, params = self.get_request_url_params(tenant, endpoint, include_related)
-        async with aiohttp.ClientSession(headers=headers) as session, session.get(
-            url, params=params
-        ) as response:
-            try:
+        proxies = self._get_proxies(self.connection) or {}
+
+        async with aiohttp.ClientSession(headers=headers) as session:
+            proxy = proxies.get("https") if proxies and url.startswith("https") else proxies.get("http")
+            extra_request_args = {}
+
+            if proxy:
+                extra_request_args["proxy"] = proxy
+
+            async with session.get(url, params=params, **extra_request_args) as response:  # type: ignore[arg-type]
                 response.raise_for_status()
                 return await response.json()
-            except ClientResponseError as e:
-                raise AirflowException(f"{e.status}:{e.message}")
 
     async def get_job_status(
         self, run_id: int, account_id: int | None = None, include_related: list[str] | None = None
@@ -277,8 +290,11 @@ class DbtCloudHook(HttpHook):
 
         return session
 
-    def _paginate(self, endpoint: str, payload: dict[str, Any] | None = None) -> list[Response]:
-        response = self.run(endpoint=endpoint, data=payload)
+    def _paginate(
+        self, endpoint: str, payload: dict[str, Any] | None = None, proxies: dict[str, str] | None = None
+    ) -> list[Response]:
+        extra_options = {"proxies": proxies} if proxies is not None else None
+        response = self.run(endpoint=endpoint, data=payload, extra_options=extra_options)
         resp_json = response.json()
         limit = resp_json["extra"]["filters"]["limit"]
         num_total_results = resp_json["extra"]["pagination"]["total_count"]
@@ -289,7 +305,7 @@ class DbtCloudHook(HttpHook):
             _paginate_payload["offset"] = limit
 
             while num_current_results < num_total_results:
-                response = self.run(endpoint=endpoint, data=_paginate_payload)
+                response = self.run(endpoint=endpoint, data=_paginate_payload, extra_options=extra_options)
                 resp_json = response.json()
                 results.append(response)
                 num_current_results += resp_json["extra"]["pagination"]["count"]
@@ -307,17 +323,20 @@ class DbtCloudHook(HttpHook):
     ) -> Any:
         self.method = method
         full_endpoint = f"api/{api_version}/accounts/{endpoint}" if endpoint else None
+        proxies = self._get_proxies(self.connection)
+        extra_options = {"proxies": proxies} if proxies is not None else None
 
         if paginate:
             if isinstance(payload, str):
                 raise ValueError("Payload cannot be a string to paginate a response.")
 
             if full_endpoint:
-                return self._paginate(endpoint=full_endpoint, payload=payload)
+                return self._paginate(endpoint=full_endpoint, payload=payload, proxies=proxies)
 
             raise ValueError("An endpoint is needed to paginate a response.")
 
-        return self.run(endpoint=full_endpoint, data=payload)
+        # breakpoint()
+        return self.run(endpoint=full_endpoint, data=payload, extra_options=extra_options)
 
     def list_accounts(self) -> list[Response]:
         """
@@ -401,6 +420,7 @@ class DbtCloudHook(HttpHook):
         account_id: int | None = None,
         steps_override: list[str] | None = None,
         schema_override: str | None = None,
+        retry_from_failure: bool = False,
         additional_run_config: dict[str, Any] | None = None,
     ) -> Response:
         """
@@ -413,6 +433,9 @@ class DbtCloudHook(HttpHook):
             instead of those configured in dbt Cloud.
         :param schema_override: Optional. Override the destination schema in the configured target for this
             job.
+        :param retry_from_failure: Optional. If set to True and the previous job run has failed, the job
+            will be triggered using the "rerun" endpoint. This parameter cannot be used alongside
+            steps_override, schema_override, or additional_run_config.
         :param additional_run_config: Optional. Any additional parameters that should be included in the API
             request when triggering the job.
         :return: The request response.
@@ -420,6 +443,15 @@ class DbtCloudHook(HttpHook):
         if additional_run_config is None:
             additional_run_config = {}
 
+        if cause is not None and len(cause) > DBT_CAUSE_MAX_LENGTH:
+            warnings.warn(
+                f"Cause `{cause}` exceeds limit of {DBT_CAUSE_MAX_LENGTH}"
+                f" characters and will be truncated.",
+                UserWarning,
+                stacklevel=2,
+            )
+            cause = cause[:DBT_CAUSE_MAX_LENGTH]
+
         payload = {
             "cause": cause,
             "steps_override": steps_override,
@@ -427,6 +459,24 @@ class DbtCloudHook(HttpHook):
         }
         payload.update(additional_run_config)
 
+        if retry_from_failure:
+            latest_run = self.get_job_runs(
+                account_id=account_id,
+                payload={
+                    "job_definition_id": job_id,
+                    "order_by": "-created_at",
+                    "limit": 1,
+                },
+            ).json()["data"]
+            if latest_run and latest_run[0]["status"] == DbtCloudJobRunStatus.ERROR.value:
+                if steps_override is not None or schema_override is not None or additional_run_config != {}:
+                    warnings.warn(
+                        "steps_override, schema_override, or additional_run_config will be ignored when"
+                        " retry_from_failure is True and previous job run has failed.",
+                        UserWarning,
+                        stacklevel=2,
+                    )
+                return self.retry_failed_job_run(job_id, account_id)
         return self._run_and_get_response(
             method="POST",
             endpoint=f"{account_id}/jobs/{job_id}/run/",
@@ -650,6 +700,17 @@ class DbtCloudHook(HttpHook):
         results = await asyncio.gather(*tasks.values())
         return {filename: result.json() for filename, result in zip(tasks.keys(), results)}
 
+    @fallback_to_default_account
+    def retry_failed_job_run(self, job_id: int, account_id: int | None = None) -> Response:
+        """
+        Retry a failed run for a job from the point of failure, if the run failed. Otherwise, trigger a new run.
+
+        :param job_id: The ID of a dbt Cloud job.
+        :param account_id: Optional. The ID of a dbt Cloud account.
+        :return: The request response.
+        """
+        return self._run_and_get_response(method="POST", endpoint=f"{account_id}/jobs/{job_id}/rerun/")
+
     def test_connection(self) -> tuple[bool, str]:
         """Test dbt Cloud connection."""
         try:
